%define anolis_release 6
%define googletest_ver 5ec7f0c4a113e2f18ac2c6cc7df51ad6afc24081
%bcond_without python
%bcond_with java

%global gtest_url https://github.com/google/googletest
%global gtest_commit 5ec7f0c4a113e2f18ac2c6cc7df51ad6afc24081
%global gtest_dir googletest-%{gtest_commit}

Name:		protobuf
Version:	3.19.6
Release:	%{anolis_release}%{?dist}
Summary:	Protocol Buffers - Google's data interchange format

License:	BSD
URL:		https://github.com/protocolbuffers/protobuf
Source0: 	https://github.com/protocolbuffers/protobuf/archive/refs/tags/v%{version}.tar.gz

Source1:        ftdetect-proto.vim
Source2:        protobuf-init.el

# For tests (using exactly the same version as the release)
Source3:        %{gtest_url}/archive/%{gtest_commit}/%{gtest_dir}.tar.gz

# Man page hand-written for Fedora in groff_man(7) format based on “protoc
# --help” output.
Source4:        protoc.1

Patch3:         protobuf-3.19.4-jre17-add-opens.patch
BuildRequires:  libtool
BuildRequires:  pkgconfig
BuildRequires:  zlib-devel
BuildRequires:  emacs  emacs-common
BuildRequires:	make autoconf automake gcc-c++

%description
Protobuf, short for Protocol Buffers, is a data serialization protocol. It was
developed by Google and was released as an open-source project in 2008. It is
designed to provide a simple and efficient way to serialize structured data for
communication between different systems.

Protobuf uses a language-neutral data format that can be used to serialize data
in a variety of programming languages such as C++, Java, Python, and Go. This
allows developers to easily share data between different systems written in
different languages.

The main advantages of using Protobuf are:

Efficiency: Protobuf generates smaller and faster messages than other
serialization formats such as XML and JSON. This is because Protobuf uses binary
encoding of data.

Flexibility: Protobuf provides a flexible schema definition language that
allows developers to define complex data structures such as nested objects and
arrays.

Interoperability: Protobuf provides language-neutral data format that can be
used by different systems written in different languages.

Extensibility: Protobuf allows developers to add new fields to the schema
without breaking compatibility with existing systems.

The basic concept of Protobuf is that data is defined in a .proto file, which
is a plain text file that contains the schema definition for the data. The
schema definition includes the data types, fields, and message formats. Once
the schema is defined, a code generator is used to generate source code for the
desired programming language.

Protobuf messages are composed of fields, each with a unique identifier and a
data type. The data types include basic types such as integers, booleans, and
strings, as well as complex types such as nested objects and arrays.

Protobuf messages can be encoded and decoded using the provided library for the
desired programming language. The encoded message can then be sent over the
network or saved to disk.

In summary, Protobuf is a simple and efficient way to serialize structured
data for communication between different systems. Its efficiency, flexibility,
interoperability, and extensibility make it a popular choice for developers who
need to share data between different systems written in different languages.
	
%package	compiler
Summary:        Protocol Buffers compiler
Requires:       %{name} = %{version}-%{release}
 
%description	compiler
This package contains Protocol Buffers compiler for all programming
languages

%package	static
Summary:        Static development files for %{name}
Requires:       %{name}-devel = %{version}-%{release}
 
%description	static
Static libraries for Protocol Buffers
 
%package	lite
Summary:        Protocol Buffers LITE_RUNTIME libraries

%description	lite
Protocol Buffers built with optimize_for = LITE_RUNTIME.
 
The "optimize_for = LITE_RUNTIME" option causes the compiler to generate code
which only depends libprotobuf-lite, which is much smaller than libprotobuf but
lacks descriptors, reflection, and some other features.
  
%package	lite-static
Summary:        Static development files for %{name}-lite
Requires:       %{name}-devel = %{version}-%{release}
 
%description	lite-static
This package contains static development libraries built with
optimize_for = LITE_RUNTIME.

The "optimize_for = LITE_RUNTIME" option causes the compiler to generate code
which only depends libprotobuf-lite, which is much smaller than libprotobuf but
lacks descriptors, reflection, and some other features.

%package	lite-devel
Summary:        Protocol Buffers LITE_RUNTIME development libraries
Requires:       %{name}-devel = %{version}-%{release}
Requires:       %{name}-lite = %{version}-%{release}
 
%description	lite-devel
This package contains development libraries built with
optimize_for = LITE_RUNTIME.
 
The "optimize_for = LITE_RUNTIME" option causes the compiler to generate code
which only depends libprotobuf-lite, which is much smaller than libprotobuf but
lacks descriptors, reflection, and some other features.

%if %{with python}
%package -n 	python3-%{name}
Summary:        Python 3 bindings for Google Protocol Buffers
BuildArch:      noarch
BuildRequires:  python3-devel
BuildRequires:  python3-setuptools
BuildRequires:  python3-wheel
Requires:       python3-six >= 1.9
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
Provides:       %{name}-python3 = %{version}-%{release}
%{?python_provide:%python_provide python3-%{name}}
 
%description -n	python3-%{name}
This package contains Python 3 libraries for Google Protocol Buffers
%endif

%package	vim
Summary:        Vim syntax highlighting for Google Protocol Buffers descriptions
BuildArch:      noarch
Requires:       vim-enhanced
 
%description	vim
This package contains syntax highlighting for Google Protocol Buffers
descriptions in Vim editor

%package	emacs
Summary:        Emacs mode for Google Protocol Buffers descriptions
BuildArch:      noarch
Requires:       emacs-filesystem >= %{_emacs_version}
Obsoletes:      protobuf-emacs-el < 3.6.1-4
 
%description	emacs
This package contains syntax highlighting for Google Protocol Buffers
descriptions in the Emacs editor.

%if %{with java}
%package	java
Summary:        Java Protocol Buffers runtime library
BuildArch:      noarch
BuildRequires:  maven-local
BuildRequires:  mvn(com.google.code.gson:gson)
BuildRequires:  mvn(com.google.guava:guava)
BuildRequires:  mvn(com.google.guava:guava-testlib)
BuildRequires:  mvn(com.google.truth:truth)
BuildRequires:  mvn(junit:junit)
BuildRequires:  mvn(org.apache.felix:maven-bundle-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-antrun-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-source-plugin)
BuildRequires:  mvn(org.codehaus.mojo:build-helper-maven-plugin)
BuildRequires:  mvn(org.easymock:easymock)
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
Obsoletes:      %{name}-javanano < 3.6.0
 
%description	java
This package contains Java Protocol Buffers runtime library.
 
%package	javalite
Summary:        Java Protocol Buffers lite runtime library
BuildArch:      noarch
 
%description	javalite
This package contains Java Protocol Buffers lite runtime library.
 
%package	java-util
Summary:        Utilities for Protocol Buffers
BuildArch:      noarch
 
%description	java-util
Utilities to work with protos. It contains JSON support
as well as utilities to work with proto3 well-known types.
 
%package	javadoc
Summary:        Javadoc for %{name}-java
BuildArch:      noarch
 
%description	javadoc
This package contains the API documentation for %{name}-java.
 
%package	parent
Summary:        Protocol Buffer Parent POM
BuildArch:      noarch
 
%description	parent
Protocol Buffer Parent POM.
 
%package	bom
Summary:        Protocol Buffer BOM POM
BuildArch:      noarch
 
%description	bom
Protocol Buffer BOM POM.

%endif

%package	devel
Summary:        Protocol Buffers C++ headers and libraries
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-compiler = %{version}-%{release}
Requires:       zlib-devel
Requires:       pkgconfig
 
%description 	devel
This package contains Protocol Buffers compiler for all languages and
C++ headers and libraries

%package doc
Summary: Doc files for %{name}
Requires: %{name} = %{EVR}
BuildArch: noarch

%description doc
The %{name}-doc package contains documentation files for %{name}

%prep
%setup -q -n %{name}-%{version} -a3
%patch3 -p1 -b .jre17

# Copy in the needed gtest/gmock implementations.
%setup -q -T -D -b 3 -n %{name}-%{version}
rm -rvf 'third_party/googletest'
mv '../%{gtest_dir}' 'third_party/googletest'

find -name \*.cc -o -name \*.h | xargs chmod -x
chmod 644 examples/*

%if %{with java}
%pom_remove_dep com.google.errorprone:error_prone_annotations java/util/pom.xml
%pom_remove_dep com.google.j2objc:j2objc-annotations java/util/pom.xml

# Remove annotation libraries we don't have
annotations=$(
    find -name '*.java' |
      xargs grep -h -e '^import com\.google\.errorprone\.annotation' \
                    -e '^import com\.google\.j2objc\.annotations' |
      sort -u | sed 's/.*\.\([^.]*\);/\1/' | paste -sd\|
)
find -name '*.java' | xargs sed -ri \
    "s/^import .*\.($annotations);//;s/@($annotations)"'\>\s*(\((("[^"]*")|([^)]*))\))?//g'

# Make OSGi dependency on sun.misc package optional
%pom_xpath_inject "pom:configuration/pom:instructions" "<Import-Package>sun.misc;resolution:=optional,*</Import-Package>" java/core
 
# Backward compatibility symlink
%mvn_file :protobuf-java:jar: %{name}/%{name}-java %{name}

%endif

rm -f src/solaris/libstdc++.la

%build
iconv -f iso8859-1 -t utf-8 CONTRIBUTORS.txt > CONTRIBUTORS.txt.utf8
mv CONTRIBUTORS.txt.utf8 CONTRIBUTORS.txt
export PTHREAD_LIBS="-lpthread"
./autogen.sh
%configure

# -Wno-error=type-limits:
#     https://github.com/protocolbuffers/protobuf/issues/7514
#     https://gcc.gnu.org/bugzilla/show_bug.cgi?id=95148
#  (also set in %%check)
%make_build CXXFLAGS="%{build_cxxflags} -Wno-error=type-limits"

%if %{with python}
pushd python
%py3_build
popd
%endif

%if %{with java}
%pom_disable_module kotlin java/pom.xml
%pom_disable_module kotlin-lite java/pom.xml
%mvn_build -s -- -f java/pom.xml
%endif

%{_emacs_bytecompile} editors/protobuf-mode.el

%install
%make_install STRIPBINARIES=no INSTALL="%{__install} -p" CPPROG="cp -p"

# protoc.1 man page
install -p -m 0644 -D -t '%{buildroot}%{_mandir}/man1' %{SOURCE4}
 
%if %{with python}
pushd python
%py3_install
find %{buildroot}%{python3_sitelib} -name \*.py |
  xargs sed -i -e '1{\@^#!@d}'
popd
%endif
install -p -m 0644 -D %{SOURCE1} %{buildroot}%{_datadir}/vim/vimfiles/ftdetect/proto.vim
install -p -m 0644 -D editors/proto.vim %{buildroot}%{_datadir}/vim/vimfiles/syntax/proto.vim


%if %{with java}
%mvn_install
%endif

mkdir -p %{buildroot}%{_emacs_sitelispdir}/%{name}
install -m 0644 -p editors/protobuf-mode.el %{buildroot}%{_emacs_sitelispdir}/%{name}
install -m 0644 -p editors/protobuf-mode.elc %{buildroot}%{_emacs_sitelispdir}/%{name}
mkdir -p %{buildroot}%{_emacs_sitestartdir}
install -m 0644 -p %{SOURCE2} %{buildroot}%{_emacs_sitestartdir}

%generate_compatibility_deps

%check
fail=1
%make_build check CXXFLAGS="%{build_cxxflags} -Wno-error=type-limits" || exit $fail

%files
%dir %{abidir}
%{_libdir}/libprotobuf.so.30*
%{abidir}/libprotobuf.dump
%{abidir}/protoc-option.list
%license LICENSE

%files	compiler
%doc README.md
%license LICENSE
%{_bindir}/protoc
%{_mandir}/man1/protoc.1*
%{_libdir}/libprotoc.so.30*
%{abidir}/libprotoc.dump

%files devel
%{_libdir}/libprotoc.so
%{_libdir}/libprotobuf.so
%dir %{_includedir}/google
%{_includedir}/google/protobuf/
%{_libdir}/pkgconfig/protobuf.pc
%doc examples/add_person.cc examples/addressbook.proto 
%doc examples/list_people.cc examples/Makefile examples/README.md

%files emacs
%{_emacs_sitelispdir}/%{name}/
%{_emacs_sitestartdir}/protobuf-init.el

%files static
%{_libdir}/libprotoc.a
%{_libdir}/libprotobuf.a

%files lite
%{_libdir}/libprotobuf-lite.so.30*
%{abidir}/libprotobuf-lite.dump

%files lite-devel
%{_libdir}/libprotobuf-lite.so
%{_libdir}/pkgconfig/protobuf-lite.pc
 
%files lite-static
%{_libdir}/libprotobuf-lite.a

%if %{with python}
%files -n python3-protobuf
%doc python/README.md
%dir %{python3_sitelib}/google
%{python3_sitelib}/google/protobuf/
%{python3_sitelib}/protobuf-%{version}%{?rcver}-py3.*.egg-info/
%{python3_sitelib}/protobuf-%{version}%{?rcver}-py3.*-nspkg.pth
%doc examples/add_person.py examples/list_people.py examples/addressbook.proto
%endif

%files vim
%{_datadir}/vim/vimfiles/syntax/proto.vim
%{_datadir}/vim/vimfiles/ftdetect/proto.vim
 
%if %{with java}
%files java -f .mfiles-protobuf-java
%license LICENSE
%doc examples/AddPerson.java examples/ListPeople.java
%doc java/README.md
 
%files java-util -f .mfiles-protobuf-java-util
 
%files javadoc -f .mfiles-javadoc
%license LICENSE
 
%files parent -f .mfiles-protobuf-parent
%license LICENSE
 
%files bom -f .mfiles-protobuf-bom
%license LICENSE
 
%files javalite -f .mfiles-protobuf-javalite
%license LICENSE
%endif

%files doc
%doc CHANGES.txt CONTRIBUTORS.txt README.md

%changelog
* Wed Mar 13 2024 Zhao Hang <wb-zh951434@alibaba-inc.com> - 3.19.6-6
- Rebuild with python3.11

* Tue Dec 26 2023 happy_orange <songnannan@linux.alibaba.com> - 3.19.6-5
- fix the source url error

* Wed Apr 12 2023 Xuchun Shang <xuchun.shang@linux.alibaba.com> - 3.19.6-4
- Add doc package and update some description

* Tue Mar 28 2023 mgb01105731 <mgb01105731@alibaba-inc.com> - 3.19.6-3
- Del Specific Fields

* Tue Mar 28 2023 Chunmei Xu <xuchunmei@linux.alibaba.com> - 3.19.6-2
- build without java

* Fri Jan 20 2023 Funda Wang <fundawang@yeah.net> - 3.19.6-1
- New version 3.19.6

* Wed Apr 27 2022 happy_orange <songnannan@linux.alibaba.com> - 3.19.4-1
- init from upstream
